"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const config_1 = require("./config");
class SuffixIndex {
    constructor(index) {
        this.index = index;
    }
    /** Look up one or more substrings, return the set matching both */
    queryPhrase(q) {
        const words = config_1.config.tokenize(q);
        const wordMatches = [];
        words.forEach(word => {
            wordMatches.push(this.queryWord(word));
        });
        // For our intersect, sort the shortest result set first
        wordMatches.sort((a, b) => a.length - b.length);
        const result = [];
        const [first, ...rest] = wordMatches;
        Object.keys(first).forEach((key) => {
            let score = first[key];
            for (const rx of rest) {
                if (!(key in rx)) {
                    return;
                }
                score *= rx[key];
            }
            result.push({ key, score });
        });
        result.sort((a, b) => b.score - a.score);
        return result;
    }
    /** Look up a single word in the index */
    queryWord(q) {
        q = q.toLowerCase();
        let current = this.index;
        for (const char of q) {
            if (!current[char]) {
                return {};
            }
            current = current[char];
        }
        const results = this.reduce(current, {});
        return results;
        //      return results.sort((a: any, b: any) => {
        //      return (Object as any).values(b)[0] - (Object as any).values(a)[0];
        //  });
    }
    reduce(r, s, parentKey = "") {
        Object.keys(r).forEach(key => {
            if (parentKey === "$") {
                s[key] = Math.max(s[key] || 0, r[key]);
            }
            else {
                this.reduce(r[key], s, key);
            }
        });
        return s;
    }
}
exports.SuffixIndex = SuffixIndex;
