"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const config_1 = require("./config");
const suffixindex_1 = require("./suffixindex");
/** A tiny substring search module
 * https://en.wikipedia.org/wiki/Suffix_tree
 * Can be optimized massively especially for space, but trading size for developer hours.
 */
class SuffixTree {
    constructor(json = {}) {
        this.index = json;
    }
    /** Serialize the tree to string */
    toJson() {
        return JSON.stringify(this.index);
    }
    /** Builds the qeryable index. */
    buildIndex() {
        return new suffixindex_1.SuffixIndex(this.index);
    }
    /** index a sequence of words */
    addSentence(sentence, searchResult) {
        if (!sentence) {
            return;
        }
        const words = config_1.config.tokenize(sentence);
        for (const word of words) {
            this.addWord(word, searchResult);
        }
    }
    /** index a single word */
    addWord(word, searchResult) {
        if (word.length < 2) {
            return;
        }
        for (let i = 0; i < word.length; i++) {
            this.addSuffix(word.substring(i).toLowerCase(), searchResult);
        }
    }
    /** index ending of a string */
    addSuffix(sub, searchResult) {
        let current = this.index;
        for (const char of sub) {
            if (!current[char]) {
                current[char] = {};
            }
            current = current[char];
        }
        if (!current.$) {
            current.$ = {};
        }
        current.$[searchResult.key] = searchResult.score;
    }
}
exports.SuffixTree = SuffixTree;
